@@ -789,6 +789,8 @@ bool BufferView::moveToPosition(pit_type bottom_pit, pos_type bottom_pos,
 		setCursor(dit);
 		// set the current font.
 		d->cursor_.setCurrentFont();
+		// Do not forget to reset the anchor (see #9912)
+		d->cursor_.resetAnchor();
 		// To center the screen on this new position we need the
 		// paragraph position which is computed at draw() time.
 		// So we need a redraw!
@@ -1223,6 +1225,7 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 
 	string const argument = to_utf8(cmd.argument());
 	Cursor & cur = d->cursor_;
+	Cursor old = cur;
 
 	// Don't dispatch function that does not apply to internal buffers.
 	if (buffer_.isInternal() 
@@ -1720,7 +1723,6 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 			p = Point(0, 0);
 		if (act == LFUN_SCREEN_DOWN && scrolled < height_)
 			p = Point(width_, height_);
-		Cursor old = cur;
 		bool const in_texted = cur.inTexted();
 		cur.setCursor(doc_iterator_begin(cur.buffer()));
 		cur.selHandle(false);
@@ -1833,6 +1835,7 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 			cur.pit() = cur.lastpit();
 			cur.pos() = cur.lastpos();
 		}
+		cur.setCurrentFont();
 		dr.screenUpdate(Update::Force);
 		break;
 
@@ -2042,6 +2045,21 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 
 	buffer_.undo().endUndoGroup();
 	dr.dispatched(dispatched);
+
+	// NOTE: The code below is copied from Cursor::dispatch. If you
+	// need to modify this, please update the other one too.
+
+	// notify insets we just entered/left
+	if (cursor() != old) {
+		old.beginUndoGroup();
+		old.fixIfBroken();
+		bool badcursor = notifyCursorLeavesOrEnters(old, cursor());
+		if (badcursor) {
+			cursor().fixIfBroken();
+			resetInlineCompletionPos();
+		}
+		old.endUndoGroup();
+	}
 }
 
 
@@ -2225,8 +2243,6 @@ void BufferView::mouseEventDispatch(FuncRequest const & cmd0)
 	if (!inset || !cur.result().dispatched())
 		cur.dispatch(cmd);
 
-	cur.endUndoGroup();
-
 	// Notify left insets
 	if (cur != old) {
 		bool badcursor = old.fixIfBroken() | cur.fixIfBroken();
@@ -2234,7 +2250,9 @@ void BufferView::mouseEventDispatch(FuncRequest const & cmd0)
 		if (badcursor)
 			cursor().fixIfBroken();
 	}
-	
+
+	cur.endUndoGroup();
+
 	// Do we have a selection?
 	theSelection().haveSelection(cursor().selection());
 
@@ -2443,6 +2461,7 @@ void BufferView::setCursor(DocIterator const & dit)
 
 	d->cursor_.setCursor(dit);
 	d->cursor_.setSelection(false);
+	d->cursor_.setCurrentFont();
 	// FIXME
 	// It seems on general grounds as if this is probably needed, but
 	// it is not yet clear.
