@@ -132,7 +132,7 @@ bool bruteFind(Cursor & cursor,
 	// Get an iterator after the last paragraph in the cache
 	DocIterator et(inset);
 	et.push_back(CursorSlice(inset));
-	et.pit() = boost::prior(cache.end())->first;
+	et.pit() = lyx::prev(cache.end(), 1)->first;
 	if (et.pit() >= et.lastpit())
 		et = doc_iterator_end(inset);
 	else
@@ -454,6 +454,9 @@ void Cursor::dispatch(FuncRequest const & cmd0)
 	}
 	endUndoGroup();
 
+	// NOTE: The code below has been copied to BufferView::dispatch.
+	// If you need to modify this, please update the other one too.
+
 	// notify insets we just left
 	if (*this != old) {
 		old.beginUndoGroup();
@@ -729,10 +732,7 @@ bool Cursor::posVisRight(bool skip_inset)
 
 	}
 
-	bool moved = (new_cur.pos() != pos()
-				  || new_cur.pit() != pit()
-				  || new_cur.boundary() != boundary()
-				  || &new_cur.inset() != &inset());
+	bool const moved = new_cur != *this || new_cur.boundary() != boundary();
 
 	if (moved) {
 		LYXERR(Debug::RTL, "moving to: " << new_cur.pos()
@@ -829,9 +829,7 @@ bool Cursor::posVisLeft(bool skip_inset)
 
 	}
 
-	bool moved = (new_cur.pos() != pos()
-				  || new_cur.pit() != pit()
-				  || new_cur.boundary() != boundary());
+	bool const moved = new_cur != *this || new_cur.boundary() != boundary();
 
 	if (moved) {
 		LYXERR(Debug::RTL, "moving to: " << new_cur.pos()
