@@ -60,13 +60,13 @@
 #include "support/lassert.h"
 #include "support/limited_stack.h"
 #include "support/lstrings.h"
+#include "support/lyxalgo.h"
 
 #include "frontends/alert.h"
 #include "frontends/Clipboard.h"
 #include "frontends/Selection.h"
 
 #include <boost/tuple/tuple.hpp>
-#include <boost/next_prior.hpp>
 
 #include <string>
 
@@ -383,7 +383,7 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 
 	// Paste it!
 	if (empty) {
-		pars.insert(boost::next(pars.begin(), pit),
+		pars.insert(lyx::next(pars.begin(), pit),
 			    insertion.begin(),
 			    insertion.end());
 
@@ -391,7 +391,7 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 		mergeParagraph(buffer.params(), pars,
 			       pit + insertion.size() - 1);
 	} else {
-		pars.insert(boost::next(pars.begin(), pit + 1),
+		pars.insert(lyx::next(pars.begin(), pit + 1),
 			    insertion.begin(),
 			    insertion.end());
 
@@ -592,8 +592,8 @@ void copySelectionHelper(Buffer const & buf, Text const & text,
 	LASSERT(startpit != endpit || start <= end, return);
 
 	// Clone the paragraphs within the selection.
-	ParagraphList copy_pars(boost::next(pars.begin(), startpit),
-				boost::next(pars.begin(), endpit + 1));
+	ParagraphList copy_pars(lyx::next(pars.begin(), startpit),
+				lyx::next(pars.begin(), endpit + 1));
 
 	// Remove the end of the last paragraph; afterwards, remove the
 	// beginning of the first paragraph. Keep this order - there may only
@@ -1274,31 +1274,43 @@ void eraseSelection(Cursor & cur)
 	//lyxerr << "cap::eraseSelection begin: " << cur << endl;
 	CursorSlice const & i1 = cur.selBegin();
 	CursorSlice const & i2 = cur.selEnd();
-	if (i1.inset().asInsetMath()) {
-		saveSelection(cur);
-		cur.top() = i1;
-		if (i1.idx() == i2.idx()) {
-			i1.cell().erase(i1.pos(), i2.pos());
-			// We may have deleted i1.cell(cur.pos()).
-			// Make sure that pos is valid.
-			if (cur.pos() > cur.lastpos())
-				cur.pos() = cur.lastpos();
-		} else {
-			InsetMath * p = i1.asInsetMath();
-			Inset::row_type r1, r2;
-			Inset::col_type c1, c2;
-			region(i1, i2, r1, r2, c1, c2);
-			for (Inset::row_type row = r1; row <= r2; ++row)
-				for (Inset::col_type col = c1; col <= c2; ++col)
-					p->cell(p->index(row, col)).clear();
-			// We've deleted the whole cell. Only pos 0 is valid.
-			cur.pos() = 0;
-		}
-		// need a valid cursor. (Lgb)
-		cur.clearSelection();
-	} else {
-		lyxerr << "can't erase this selection 1" << endl;
+	if (!i1.asInsetMath()) {
+		LYXERR0("Can't erase this selection");
+		return;
 	}
+
+	saveSelection(cur);
+	cur.top() = i1;
+	InsetMath * p = i1.asInsetMath();
+	if (i1.idx() == i2.idx()) {
+		i1.cell().erase(i1.pos(), i2.pos());
+		// We may have deleted i1.cell(cur.pos()).
+		// Make sure that pos is valid.
+		if (cur.pos() > cur.lastpos())
+			cur.pos() = cur.lastpos();
+	} else if (p->nrows() > 0 && p->ncols() > 0) {
+		// This is a grid, delete a nice square region
+		Inset::row_type r1, r2;
+		Inset::col_type c1, c2;
+		region(i1, i2, r1, r2, c1, c2);
+		for (Inset::row_type row = r1; row <= r2; ++row)
+			for (Inset::col_type col = c1; col <= c2; ++col)
+				p->cell(p->index(row, col)).clear();
+		// We've deleted the whole cell. Only pos 0 is valid.
+		cur.pos() = 0;
+	} else {
+		Inset::idx_type idx1 = i1.idx();
+		Inset::idx_type idx2 = i2.idx();
+		if (idx1 > idx2)
+			swap(idx1, idx2);
+		for (Inset::idx_type idx = idx1 ; idx <= idx2; ++idx)
+			p->cell(idx).clear();
+		// We've deleted the whole cell. Only pos 0 is valid.
+		cur.pos() = 0;
+	}
+
+	// need a valid cursor. (Lgb)
+	cur.clearSelection();
 	//lyxerr << "cap::eraseSelection end: " << cur << endl;
 }
 
