@@ -66,10 +66,9 @@
 #include "support/gettext.h"
 #include "support/lassert.h"
 #include "support/lstrings.h"
+#include "support/lyxalgo.h"
 #include "support/textutils.h"
 
-#include <boost/next_prior.hpp>
-
 #include <limits>
 #include <sstream>
 
@@ -137,7 +136,7 @@ void breakParagraphConservative(BufferParams const & bparams,
 	ParagraphList & pars, pit_type par_offset, pos_type pos)
 {
 	// create a new paragraph
-	Paragraph & tmp = *pars.insert(boost::next(pars.begin(), par_offset + 1),
+	Paragraph & tmp = *pars.insert(lyx::next(pars.begin(), par_offset + 1),
 				       Paragraph());
 	Paragraph & par = pars[par_offset];
 
@@ -193,7 +192,7 @@ void mergeParagraph(BufferParams const & bparams,
 	// move the change of the end-of-paragraph character
 	par.setChange(par.size(), change);
 
-	pars.erase(boost::next(pars.begin(), par_offset + 1));
+	pars.erase(lyx::next(pars.begin(), par_offset + 1));
 }
 
 
@@ -639,7 +638,7 @@ static void breakParagraph(Text & text, pit_type par_offset, pos_type pos,
 	ParagraphList & pars = text.paragraphs();
 	// create a new paragraph, and insert into the list
 	ParagraphList::iterator tmp =
-		pars.insert(boost::next(pars.begin(), par_offset + 1),
+		pars.insert(lyx::next(pars.begin(), par_offset + 1),
 			    Paragraph());
 
 	Paragraph & par = pars[par_offset];
@@ -1603,14 +1602,14 @@ bool Text::backspacePos0(Cursor & cur)
 	if (cur.lastpos() == 0
 	    || (cur.lastpos() == 1 && par.isSeparator(0))) {
 		cur.recordUndo(ATOMIC_UNDO, prevcur.pit(), cur.pit());
-		plist.erase(boost::next(plist.begin(), cur.pit()));
+		plist.erase(lyx::next(plist.begin(), cur.pit()));
 		needsUpdate = true;
 	}
 	// is previous par empty?
 	else if (prevcur.lastpos() == 0
 		 || (prevcur.lastpos() == 1 && prevpar.isSeparator(0))) {
 		cur.recordUndo(ATOMIC_UNDO, prevcur.pit(), cur.pit());
-		plist.erase(boost::next(plist.begin(), prevcur.pit()));
+		plist.erase(lyx::next(plist.begin(), prevcur.pit()));
 		needsUpdate = true;
 	}
 	// Pasting is not allowed, if the paragraphs have different
