@@ -50,10 +50,9 @@
 #include "support/lassert.h"
 #include "support/debug.h"
 #include "support/gettext.h"
+#include "support/lyxalgo.h"
 #include "support/textutils.h"
 
-#include <boost/next_prior.hpp>
-
 #include <sstream>
 
 using namespace std;
@@ -396,7 +395,7 @@ bool Text::cursorTop(Cursor & cur)
 bool Text::cursorBottom(Cursor & cur)
 {
 	LBUFERR(this == cur.text());
-	return setCursor(cur, cur.lastpit(), boost::prior(paragraphs().end())->size());
+	return setCursor(cur, cur.lastpit(), lyx::prev(paragraphs().end(), 1)->size());
 }
 
 
@@ -895,7 +894,7 @@ bool Text::deleteEmptyParagraphMechanism(Cursor & cur,
 			   min(old.pit() + 1, old.lastpit()));
 		ParagraphList & plist = old.text()->paragraphs();
 		bool const soa = oldpar.params().startOfAppendix();
-		plist.erase(boost::next(plist.begin(), old.pit()));
+		plist.erase(lyx::next(plist.begin(), old.pit()));
 		// do not lose start of appendix marker (bug 4212)
 		if (soa && old.pit() < pit_type(plist.size()))
 			plist[old.pit()].params().startOfAppendix(true);
@@ -959,7 +958,7 @@ void Text::deleteEmptyParagraphMechanism(pit_type first, pit_type last, bool tra
 			continue;
 
 		if (par.empty() || (par.size() == 1 && par.isLineSeparator(0))) {
-			pars_.erase(boost::next(pars_.begin(), pit));
+			pars_.erase(lyx::next(pars_.begin(), pit));
 			--pit;
 			--last;
 			continue;
