@@ -71,6 +71,7 @@
 #include "support/gettext.h"
 #include "support/lassert.h"
 #include "support/lstrings.h"
+#include "support/lyxalgo.h"
 #include "support/lyxtime.h"
 #include "support/os.h"
 #include "support/regex.h"
@@ -78,8 +79,6 @@
 #include "mathed/InsetMathHull.h"
 #include "mathed/MathMacroTemplate.h"
 
-#include <boost/next_prior.hpp>
-
 #include <clocale>
 #include <sstream>
 
@@ -340,7 +339,7 @@ static void outline(OutlineOp mode, Cursor & cur)
 	ParagraphList & pars = buf.text().paragraphs();
 	ParagraphList::iterator const bgn = pars.begin();
 	// The first paragraph of the area to be copied:
-	ParagraphList::iterator start = boost::next(bgn, pit);
+	ParagraphList::iterator start = lyx::next(bgn, pit);
 	// The final paragraph of area to be copied:
 	ParagraphList::iterator finish = start;
 	ParagraphList::iterator const end = pars.end();
@@ -393,7 +392,7 @@ static void outline(OutlineOp mode, Cursor & cur)
 				// Nothing to move.
 				return;
 			// Go one down from *this* header:
-			ParagraphList::iterator dest = boost::next(finish, 1);
+			ParagraphList::iterator dest = lyx::next(finish, 1);
 			// Go further down to find header to insert in front of:
 			for (; dest != end; ++dest) {
 				toclevel = buf.text().getTocLevel(distance(bgn, dest));
@@ -781,7 +780,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 		ParagraphList & pars = buf.text().paragraphs();
 		ParagraphList::iterator bgn = pars.begin();
 		// The first paragraph of the area to be selected:
-		ParagraphList::iterator start = boost::next(bgn, pit);
+		ParagraphList::iterator start = lyx::next(bgn, pit);
 		// The final paragraph of area to be selected:
 		ParagraphList::iterator finish = start;
 		ParagraphList::iterator end = pars.end();
@@ -2129,6 +2128,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 
 	case LFUN_FINISHED_BACKWARD:
 		LYXERR(Debug::DEBUG, "handle LFUN_FINISHED_BACKWARD:\n" << cur);
+		cur.setCurrentFont();
 		break;
 
 	case LFUN_FINISHED_FORWARD:
@@ -2965,13 +2965,12 @@ bool Text::getStatus(Cursor & cur, FuncRequest const & cmd,
 		break;
 
 	case LFUN_LAYOUT: {
-		enable = !cur.inset().forcePlainLayout();
-
+		DocumentClass const & tclass = cur.buffer()->params().documentClass();
 		docstring layout = cmd.argument();
-		if (layout.empty()) {
-			DocumentClass const & tclass = cur.buffer()->params().documentClass();
+		if (layout.empty())
 			layout = tclass.defaultLayoutName();
-		}
+		enable = !cur.inset().forcePlainLayout() && tclass.hasLayout(layout);
+
 		flag.setOnOff(layout == cur.paragraph().layout().name());
 		break;
 	}
