@@ -10,8 +10,9 @@
 #ifndef TEXTCLASS_H
 #define TEXTCLASS_H
 
-#include "ColorCode.h"
+#include "Citation.h"
 #include "Counters.h"
+#include "DocumentClassPtr.h"
 #include "FloatList.h"
 #include "FontInfo.h"
 #include "Layout.h"
@@ -23,8 +24,6 @@
 #include "support/docstring.h"
 #include "support/types.h"
 
-#include <boost/noncopyable.hpp>
-
 #include <list>
 #include <map>
 #include <set>
@@ -55,24 +54,24 @@ private:
 };
 
 
-/// A TextClass represents a collection of layout information: At the 
+/// A TextClass represents a collection of layout information: At the
 /// moment, this includes Layout's and InsetLayout's.
 ///
 /// There are two major subclasses of TextClass: LayoutFile and
 /// DocumentClass. These subclasses are what are actually used in LyX.
-/// Simple TextClass objects are not directly constructed in the main 
+/// Simple TextClass objects are not directly constructed in the main
 /// LyX code---the constructor is protected. (That said, in tex2lyx
 /// there are what amount to simple TextClass objects.)
 ///
 /// A LayoutFile (see LayoutFile.{h,cpp}) represents a *.layout file.
-/// These are generally static objects---though they can be reloaded 
-/// from disk via LFUN_LAYOUT_RELOAD, so one should not assume that 
+/// These are generally static objects---though they can be reloaded
+/// from disk via LFUN_LAYOUT_RELOAD, so one should not assume that
 /// they will never change.
 ///
 /// A DocumentClass (see below) represents the layout information that
 /// is associated with a given Buffer. These are static, in the sense
 /// that they will not themselves change, but which DocumentClass is
-/// associated with a Buffer can change, as modules are loaded and 
+/// associated with a Buffer can change, as modules are loaded and
 /// unloaded, for example.
 ///
 class TextClass : protected ProtectCopy {
@@ -84,7 +83,7 @@ public:
 	///////////////////////////////////////////////////////////////////
 	// NOTE Do NOT try to make this a container of Layout pointers, e.g.,
 	// std::list<Layout *>. This will lead to problems. The reason is
-	// that DocumentClass objects are generally created by copying a 
+	// that DocumentClass objects are generally created by copying a
 	// LayoutFile, which serves as a base for the DocumentClass. If the
 	// LayoutList is a container of pointers, then every DocumentClass
 	// that derives from a given LayoutFile (e.g., article) will SHARE
@@ -120,14 +119,14 @@ public:
 	docstring const & defaultLayoutName() const;
 	///
 	bool isDefaultLayout(Layout const &) const;
-	/// 
+	///
 	bool isPlainLayout(Layout const &) const;
 	/// returns a special layout for use when we don't really want one,
 	/// e.g., in table cells
-	Layout const & plainLayout() const 
+	Layout const & plainLayout() const
 			{ return operator[](plain_layout_); }
 	/// the name of the plain layout
-	docstring const & plainLayoutName() const 
+	docstring const & plainLayoutName() const
 			{ return plain_layout_; }
 	/// Enumerate the paragraph styles.
 	size_t layoutCount() const { return layoutlist_.size(); }
@@ -144,7 +143,7 @@ public:
 	// reading routines
 	///////////////////////////////////////////////////////////////////
 	/// Enum used with TextClass::read
-	enum ReadType { 
+	enum ReadType {
 		BASECLASS, //>This is a base class, i.e., top-level layout file
 		MERGE, //>This is a file included in a layout file
 		MODULE, //>This is a layout module
@@ -153,19 +152,23 @@ public:
 	/// return values for read()
 	enum ReturnValues {
 		OK,
+		OK_OLDFORMAT,
 		ERROR,
 		FORMAT_MISMATCH
 	};
 
 	/// Performs the read of the layout file.
 	/// \return true on success.
+	// FIXME Should return ReturnValues....
 	bool read(support::FileName const & filename, ReadType rt = BASECLASS);
 	///
-	bool read(std::string const & str, ReadType rt = BASECLASS);
+	ReturnValues read(std::string const & str, ReadType rt = MODULE);
 	///
 	ReturnValues read(Lexer & lex, ReadType rt = BASECLASS);
 	/// validates the layout information passed in str
-	static bool validate(std::string const & str);
+	static ReturnValues validate(std::string const & str);
+	///
+	static std::string convert(std::string const & str);
 
 	///////////////////////////////////////////////////////////////////
 	// loading
@@ -185,11 +188,13 @@ public:
 	///
 	std::string const & name() const { return name_; }
 	///
+	std::string const & category() const { return category_; }
+	///
 	std::string const & description() const { return description_; }
 	///
 	std::string const & latexname() const { return latexname_; }
 	///
-	std::string const & prerequisites() const;
+	std::string const & prerequisites(std::string const & sep = "\n\t") const;
 	/// Can be LaTeX, DocBook, etc.
 	OutputType outputType() const { return outputType_; }
 	/// Can be latex, docbook ... (the name of a format)
@@ -200,14 +205,14 @@ protected:
 	///
 	Layout & operator[](docstring const & name);
 	/** Create an new, very basic layout for this textclass. This is used for
-	    the Plain Layout common to all TextClass objects and also, in 
+	    the Plain Layout common to all TextClass objects and also, in
 	    DocumentClass, for the creation of new layouts `on the fly' when
 	    previously unknown layouts are encountered.
-	    \param unknown Set to true if this layout is used to represent an 
+	    \param unknown Set to true if this layout is used to represent an
 	    unknown layout
 	 */
 	Layout createBasicLayout(docstring const & name, bool unknown = false) const;
-	
+
 	///////////////////////////////////////////////////////////////////
 	// non-const iterators
 	///////////////////////////////////////////////////////////////////
@@ -227,6 +232,8 @@ protected:
 	mutable LayoutList layoutlist_;
 	/// Layout file name
 	std::string name_;
+	/// Class category
+	std::string category_;
 	/// document class name
 	std::string latexname_;
 	/// document class description
@@ -241,6 +248,8 @@ protected:
 	bool tex_class_avail_;
 	/// document class prerequisites
 	mutable std::string prerequisites_;
+	/// The possible cite engine types
+	std::string opt_enginetype_;
 	///
 	std::string opt_fontsize_;
 	///
@@ -260,12 +269,16 @@ protected:
 	/// same, but for HTML output
 	/// this is output as is to the header
 	docstring htmlpreamble_;
+	/// same, but specifically for CSS information
+	docstring htmlstyles_;
 	/// the paragraph style to use for TOCs, Bibliography, etc
 	mutable docstring html_toc_section_;
 	/// latex packages loaded by document class.
 	std::set<std::string> provides_;
 	/// latex packages requested by document class.
 	std::set<std::string> requires_;
+	///
+	std::map<std::string, std::string> package_options_;
 	/// default modules wanted by document class
 	LayoutModuleList default_modules_;
 	/// modules provided by document class
@@ -305,9 +318,15 @@ protected:
 	/// The maximal TocLevel of sectioning layouts
 	int max_toclevel_;
 	/// Citation formatting information
-	std::map<std::string, std::string> cite_formats_;
+	std::map<CiteEngineType, std::map<std::string, std::string> > cite_formats_;
 	/// Citation macros
-	std::map<std::string, std::string> cite_macros_;
+	std::map<CiteEngineType, std::map<std::string, std::string> > cite_macros_;
+	/// The default BibTeX bibliography style file
+	std::string cite_default_biblio_style_;
+	/// Whether full author lists are supported
+	bool cite_full_author_list_;
+	/// The possible citation styles
+	std::map<CiteEngineType, std::vector<CitationStyle> > cite_styles_;
 private:
 	///////////////////////////////////////////////////////////////////
 	// helper routines for reading layout files
@@ -315,6 +334,8 @@ private:
 	///
 	bool deleteLayout(docstring const &);
 	///
+	bool deleteInsetLayout(docstring const &);
+	///
 	bool convertLayoutFormat(support::FileName const &, ReadType);
 	/// Reads the layout file without running layout2layout.
 	ReturnValues readWithoutConv(support::FileName const & filename, ReadType rt);
@@ -333,24 +354,28 @@ private:
 	///
 	bool readFloat(Lexer &);
 	///
-	void readCiteFormat(Lexer &);
+	bool readCiteEngine(Lexer &);
+	///
+	int readCiteEngineType(Lexer &) const;
+	///
+	bool readCiteFormat(Lexer &);
 };
 
 
 /// A DocumentClass represents the layout information associated with a
 /// Buffer. It is based upon a LayoutFile, but may be modified by loading
-/// various Modules. 
-/// 
+/// various Modules.
+///
 /// In that regard, DocumentClass objects are "dynamic". But this is really
 /// an illusion, since DocumentClass objects are not (currently) changed
 /// when, say, a new Module is loaded. Rather, the old DocumentClass is
 /// discarded---actually, it's kept around in case something on the cut
-/// stack needs it---and a new one is created from scratch. 
-/// 
+/// stack needs it---and a new one is created from scratch.
+///
 /// In the main LyX code, DocumentClass objects are created only by
 /// DocumentClassBundle, for which see below.
-/// 
-class DocumentClass : public TextClass, boost::noncopyable {
+///
+class DocumentClass : public TextClass {
 public:
 	///
 	virtual ~DocumentClass() {}
@@ -371,10 +396,12 @@ public:
 	/// happen).  -- Idea JMarc, comment MV
 	InsetLayout const & insetLayout(docstring const & name) const;
 	/// a plain inset layout for use as a default
-	static InsetLayout const & plainInsetLayout() { return plain_insetlayout_; }
+	static InsetLayout const & plainInsetLayout();
 	/// add a new layout \c name if it does not exist in layoutlist_
 	/// \return whether we had to add one.
 	bool addLayoutIfNeeded(docstring const & name) const;
+	/// Forced layouts in layout file syntax
+	std::string forcedLayouts() const;
 
 	///////////////////////////////////////////////////////////////////
 	// accessors
@@ -384,6 +411,8 @@ public:
 	///
 	Counters & counters() const { return counters_; }
 	///
+	std::string const & opt_enginetype() const { return opt_enginetype_; }
+	///
 	std::string const & opt_fontsize() const { return opt_fontsize_; }
 	///
 	std::string const & opt_pagestyle() const { return opt_pagestyle_; }
@@ -398,6 +427,10 @@ public:
 	///
 	docstring const & htmlpreamble() const { return htmlpreamble_; }
 	///
+	docstring const & htmlstyles() const { return htmlstyles_; }
+	/// Looks for the layout of "highest level", other than Part (or other
+	/// layouts with a negative toc number), for use in constructing TOCs and 
+	/// similar information.
 	Layout const & getTOCLayout() const;
 	/// the paragraph style to use for TOCs, Bibliography, etc
 	/// we will attempt to calculate this if it was not given
@@ -406,6 +439,9 @@ public:
 	bool provides(std::string const & p) const;
 	/// features required by the class?
 	std::set<std::string> const & requires() const { return requires_; }
+	/// package options to write to LaTeX file
+	std::map<std::string, std::string> const & packageOptions() const
+		{ return package_options_; }
 	///
 	unsigned int columns() const { return columns_; }
 	///
@@ -433,9 +469,19 @@ public:
 	/// returns true if the class has a ToC structure
 	bool hasTocLevels() const;
 	///
-	std::string const & getCiteFormat(std::string const & entry_type) const;
+	std::string const & getCiteFormat(CiteEngineType const & type,
+		std::string const & entry, std::string const & fallback = "") const;
 	///
-	std::string const & getCiteMacro(std::string const & macro) const;
+	std::string const & getCiteMacro(CiteEngineType const & type,
+		std::string const & macro) const;
+	///
+	std::vector<std::string> const citeCommands(CiteEngineType const &) const;
+	///
+	std::vector<CitationStyle> const & citeStyles(CiteEngineType const &) const;
+	///
+	std::string const & defaultBiblioStyle() const { return cite_default_biblio_style_; }
+	///
+	bool const & fullAuthorList() const { return cite_full_author_list_; }
 protected:
 	/// Constructs a DocumentClass based upon a LayoutFile.
 	DocumentClass(LayoutFile const & tc);
@@ -444,42 +490,19 @@ protected:
 private:
 	/// The only class that can create a DocumentClass is
 	/// DocumentClassBundle, which calls the protected constructor.
-	friend class DocumentClassBundle;
-	///
-	static InsetLayout plain_insetlayout_;
+	friend DocumentClassPtr
+		getDocumentClass(LayoutFile const &, LayoutModuleList const &,
+				 bool const clone);
 };
 
 
-/// DocumentClassBundle is a container for DocumentClass objects, so that 
-/// they stay in memory for use by Insets, CutAndPaste, and the like, even
-/// when their associated Buffers are destroyed.
-/// FIXME Some sort of garbage collection or reference counting wouldn't
-/// be a bad idea here. It might be enough to check when a Buffer is closed
-/// (or makeDocumentClass is called) whether the old DocumentClass is in use 
-/// anywhere.
-///
-/// This is a singleton class. Its sole instance is accessed via 
-/// DocumentClassBundle::get().
-class DocumentClassBundle : boost::noncopyable {
-public:
-	/// \return The sole instance of this class.
-	static DocumentClassBundle & get();
-	/// \return A new DocumentClass based on baseClass, with info added
-	/// from the modules in modlist.
-	DocumentClass & makeDocumentClass(LayoutFile const & baseClass, 
+/// The only way to make a DocumentClass is to call this function.
+/// The shared_ptr is needed because DocumentClass objects can be kept
+/// in memory long after their associated Buffer is destroyed, mostly
+/// on the CutStack.
+DocumentClassPtr getDocumentClass(LayoutFile const & baseClass,
 			LayoutModuleList const & modlist,
 			bool const clone = false);
-private:
-	/// control instantiation
-	DocumentClassBundle() {}
-	/// clean up
-	~DocumentClassBundle();
-	/// \return Reference to a new DocumentClass equal to baseClass
-	DocumentClass & newClass(LayoutFile const & baseClass);
-	///
-	std::vector<DocumentClass *> documentClasses_;
-};
-
 
 /// convert page sides option to text 1 or 2
 std::ostream & operator<<(std::ostream & os, PageSides p);
