@@ -321,6 +321,10 @@ void GuiTabular::checkEnabled()
 	multirowOffsetED->setEnabled(enable_mr);
 	multirowOffsetUnitLC->setEnabled(enable_mr);
 
+	// Vertical lines cannot be set in formal tables
+	borders->setLeftEnabled(!booktabsRB->isChecked());
+	borders->setRightEnabled(!booktabsRB->isChecked());
+
 	changed();
 }
 
