@@ -51,7 +51,6 @@
 #include "support/lstrings.h"
 
 #include "support/regex.h"
-#include <boost/next_prior.hpp>
 
 using namespace std;
 using namespace lyx::support;
@@ -386,7 +385,6 @@ bool lyxreplace(BufferView * bv,
 		else
 			bv->message(_("String not found."));
 	}
-	bv->buffer().updatePreviews();
 	return update;
 }
 
@@ -1426,7 +1424,6 @@ static void findAdvReplace(BufferView * bv, FindAndReplaceOptions const & opt, M
 	LYXERR(Debug::FIND, "After pos adj cur=" << cur << " with depth: " << cur.depth() << " and len: " << sel_len);
 	bv->putSelectionAt(DocIterator(cur), sel_len, !opt.forward);
 	bv->processUpdateFlags(Update::Force);
-	bv->buffer().updatePreviews();
 }
 
 
