@@ -729,10 +729,17 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 	}
 
 	case LFUN_MOUSE_DOUBLE:
-	case LFUN_MOUSE_TRIPLE:
 	case LFUN_WORD_SELECT:
 		cur.pos() = 0;
+		cur.resetAnchor();
+		cur.setSelection(true);
+		cur.pos() = cur.lastpos();
+		cur.bv().cursor() = cur;
+		break;
+
+	case LFUN_MOUSE_TRIPLE:
 		cur.idx() = 0;
+		cur.pos() = 0;
 		cur.resetAnchor();
 		cur.setSelection(true);
 		cur.idx() = cur.lastidx();
@@ -2140,6 +2147,7 @@ MathCompletionList::MathCompletionList(Cursor const & cur)
 	globals.push_back(from_ascii("\\sqrt"));
 	globals.push_back(from_ascii("\\root"));
 	globals.push_back(from_ascii("\\tabular"));
+	globals.push_back(from_ascii("\\sideset"));
 	globals.push_back(from_ascii("\\stackrel"));
 	globals.push_back(from_ascii("\\stackrelthree"));
 	globals.push_back(from_ascii("\\binom"));
@@ -2179,6 +2187,7 @@ MathCompletionList::MathCompletionList(Cursor const & cur)
 	globals.push_back(from_ascii("\\mathclap"));
 	globals.push_back(from_ascii("\\mathllap"));
 	globals.push_back(from_ascii("\\mathrlap"));
+	globals.push_back(from_ascii("\\ensuremath"));
 	MathWordList const & words = mathedWordList();
 	MathWordList::const_iterator it2;
 	//lyxerr << "Globals completion commands: ";
