@@ -38,7 +38,7 @@
 
 #include "support/gettext.h"
 #include "support/lassert.h"
-#include <boost/next_prior.hpp>
+#include "support/lyxalgo.h"
 
 #include <cstdlib>
 
@@ -431,13 +431,8 @@ void MathData::updateMacros(Cursor * cur, MacroContext const & mc,
 		if (oldDisplayMode == MathMacro::DISPLAY_NORMAL
 		    && (macroInset->arity() != macroNumArgs
 			|| macroInset->optionals() != macroOptionals
-			|| newDisplayMode == MathMacro::DISPLAY_UNFOLDED)) {
-
+			|| newDisplayMode == MathMacro::DISPLAY_UNFOLDED))
 			detachMacroParameters(cur, i);
-			// FIXME: proper anchor handling, this removes the selection
-			if (cur)
-				cur->clearSelection();
-		}
 
 		// the macro could have been copied while resizing this
 		macroInset = operator[](i).nucleus()->asMacro();
@@ -475,12 +470,9 @@ void MathData::updateMacros(Cursor * cur, MacroContext const & mc,
 			// attach parameters
 			attachMacroParameters(cur, i, macroNumArgs, macroOptionals,
 				fromInitToNormalMode, interactive, appetite);
-			
-			if (cur) {
-				// FIXME: proper anchor handling, this removes the selection
+
+			if (cur)
 				cur->updateInsets(&cur->bottom().inset());
-				cur->clearSelection();	
-			}
 		}
 
 		// Give macro the chance to adapt to new situation.
@@ -900,7 +892,7 @@ MathData::size_type MathData::x2pos(BufferView const * bv, int targetx, int glue
 	 * See bug 1918 for details.
 	 **/
 	if (it != begin() && currx >= targetx
-	    && ((*boost::prior(it))->asNestInset()
+	    && ((*lyx::prev(it, 1))->asNestInset()
 		|| abs(lastx - targetx) < abs(currx - targetx))) {
 		--it;
 	}
